@@ -7,7 +7,7 @@ from trac.perm import IPermissionRequestor
 from trac.config import Option, ListOption
 
 revision = "$Rev: 7457 $"
-url = "$URL: https://svn.edgewall.org/repos/trac/tags/trac-0.12.2/sample-plugins/workflow/CodeReview.py $"
+url = "$URL: file:///var/svn/trac/tags/trac-0.12.3/sample-plugins/workflow/CodeReview.py $"
 
 class CodeReviewActionController(Component):
     """Support for simple code reviews.
