@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # 
-# Copyright (C) 2004-2008 Edgewall Software
+# Copyright (C) 2004-2009 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -13,22 +13,41 @@
 #
 # Author: Tim Moloney <t.moloney@verizon.net>
 
-import ConfigParser
 import difflib
 import os
 import re
-import shlex
 import sys
-import time
 import unittest
-import traceback
 from StringIO import StringIO
 
+# IAdminCommandProvider implementations
+import trac.admin.api
+import trac.attachment
+import trac.perm
+import trac.ticket.admin
+import trac.versioncontrol.admin
+import trac.versioncontrol.api
+import trac.versioncontrol.web_ui
+import trac.wiki.admin
+
+# IPermissionRequestor implementations (for 'permission' related tests)
+import trac.about
+import trac.admin.web_ui
+import trac.config
+import trac.ticket.api
+import trac.ticket.report
+import trac.ticket.roadmap
+import trac.ticket.web_ui
+import trac.search.web_ui
+import trac.timeline.web_ui
+import trac.wiki.web_ui
+
 from trac.config import Configuration
 from trac.env import Environment
-from trac.admin import console
+from trac.admin import console, console_date_format
 from trac.test import InMemoryDatabase
-from trac.util.datefmt import get_date_format_hint
+from trac.util.datefmt import format_date, get_date_format_hint
+from trac.web.tests.session import _prep_session_table
 
 STRIP_TRAILING_SPACE = re.compile(r'( +)$', re.MULTILINE)
 
@@ -54,18 +73,27 @@ def load_expected_results(file, pattern):
     return expected
 
 
+class InMemoryConfiguration(Configuration):
+    """A subclass of Configuration that doesn't save to disk."""
+    def save(self):
+        pass
+
+
 class InMemoryEnvironment(Environment):
     """
-    A subclass of Environment that keeps its' DB in memory.
+    A subclass of Environment that keeps its DB in memory.
     """
 
+    def get_read_db(self):
+        return self.get_db_cnx()
+
     def get_db_cnx(self):
         if not hasattr(self, '_db'):
             self._db = InMemoryDatabase()
         return self._db
 
-    def create(self, db_str=None):
-        pass
+    def create(self, options=[]):
+        self.setup_config()
 
     def verify(self):
         return True
@@ -78,11 +106,9 @@ class InMemoryEnvironment(Environment):
         return cls.__module__.startswith('trac.') and \
                cls.__module__.find('.tests.') == -1
 
-    def setup_config(self, load_defaults=None):
-        self.config = Configuration(None)
-
-    def save_config(self):
-        pass
+    def setup_config(self):
+        self.config = InMemoryConfiguration(None)
+        self.setup_log()
 
 
 class TracadminTestCase(unittest.TestCase):
@@ -91,9 +117,9 @@ class TracadminTestCase(unittest.TestCase):
     .../trac/tests.py.
     """
 
-    expected_results = load_expected_results(os.path.join(os.path.split(__file__)[0],
-                                            'console-tests.txt'),
-                                            '===== (test_[^ ]+) =====')
+    expected_results = load_expected_results(
+            os.path.join(os.path.split(__file__)[0], 'console-tests.txt'),
+            '===== (test_[^ ]+) =====')
 
     def setUp(self):
         self.env = InMemoryEnvironment('', create=True)
@@ -108,7 +134,7 @@ class TracadminTestCase(unittest.TestCase):
     def tearDown(self):
         self.env = None
 
-    def _execute(self, cmd, strip_trailing_space=True, expect_exception=False):
+    def _execute(self, cmd, strip_trailing_space=True):
         _err = sys.stderr
         _out = sys.stdout
         try:
@@ -119,24 +145,34 @@ class TracadminTestCase(unittest.TestCase):
                 retval = self._admin.onecmd(cmd)
             except SystemExit, e:
                 pass
-            sys.stderr = _err
-            sys.stdout = _out
             value = out.getvalue()
             if isinstance(value, str): # reverse what print_listing did
                 value = value.decode('utf-8')
+            # DEBUG: uncomment in case of `AssertionError: 0 != 2` in tests
+            #if retval != 0:
+            #    print>>_err, value
             if strip_trailing_space:
                 return retval, STRIP_TRAILING_SPACE.sub('', value)
             else:
                 return retval, value
-        except Exception, e:
+        finally:
             sys.stderr = _err
             sys.stdout = _out
-            if expect_exception:
-                tb = traceback.format_exc()
-                message = tb.splitlines()[-1] + '\n'
-                return -1, message
-            raise
 
+    def assertEqual(self, expected_results, output):
+        if not (isinstance(expected_results, basestring) and \
+                isinstance(output, basestring)):
+            return unittest.TestCase.assertEqual(self, expected_results, output)
+        # Create a useful delta between the output and the expected output
+        output_lines = ['%s\n' % x for x in output.split('\n')]
+        expected_lines = ['%s\n' % x for x in expected_results.split('\n')]
+        output_diff = ''.join(list(
+            difflib.unified_diff(expected_lines, output_lines,
+                                 'expected', 'actual')
+        ))
+        unittest.TestCase.assertEqual(self, expected_results, output, 
+                                      "%r != %r\n%s" %
+                                      (expected_results, output, output_diff))
     # Help test
 
     def test_help_ok(self):
@@ -153,14 +189,61 @@ class TracadminTestCase(unittest.TestCase):
         expected_results = self.expected_results[test_name] % d
         rv, output = self._execute('help')
         self.assertEqual(0, rv)
-        # Create a useful delta between the output and the expected output
-        output_lines = ['%s\n' % x for x in output.split('\n')]
-        expected_lines = ['%s\n' % x for x in expected_results.split('\n')]
-        output_diff = ''.join(list(
-            difflib.unified_diff(expected_lines, output_lines)
-        ))
-        failure_message = "%r != %r\n" % (output, expected_results) + output_diff
-        self.assertEqual(expected_results, output, failure_message)
+        self.assertEqual(expected_results, output)
+
+    # Attachment tests
+    
+    def test_attachment_list_empty(self):
+        """
+        Tests the 'attachment list' command in trac-admin, on a wiki page that
+        doesn't have any attachments.
+        """
+        # FIXME: Additional tests should be written for the other 'attachment'
+        #        commands. This requires being able to control the current
+        #        time, which in turn would require centralizing the time
+        #        provider, for example in the environment object.
+        test_name = sys._getframe().f_code.co_name
+        rv, output = self._execute('attachment list wiki:WikiStart')
+        self.assertEqual(0, rv)
+        self.assertEqual(self.expected_results[test_name], output)
+    
+    # Config tests
+    
+    def test_config_get(self):
+        """
+        Tests the 'config get' command in trac-admin.  This particular
+        test gets the project name from the config.
+        """
+        test_name = sys._getframe().f_code.co_name
+        self.env.config.set('project', 'name', 'Test project')
+        rv, output = self._execute('config get project name')
+        self.assertEqual(0, rv)
+        self.assertEqual(self.expected_results[test_name], output)
+
+    def test_config_set(self):
+        """
+        Tests the 'config set' command in trac-admin.  This particular
+        test sets the project name using an option value containing a space.
+        """
+        test_name = sys._getframe().f_code.co_name
+        rv, output = self._execute('config set project name "Test project"')
+        self.assertEqual(0, rv)
+        self.assertEqual(self.expected_results[test_name], output)
+        self.assertEqual('Test project',
+                         self.env.config.get('project', 'name'))
+
+    def test_config_remove(self):
+        """
+        Tests the 'config remove' command in trac-admin.  This particular
+        test removes the project name from the config, therefore reverting
+        the option to the default value.
+        """
+        test_name = sys._getframe().f_code.co_name
+        self.env.config.set('project', 'name', 'Test project')
+        rv, output = self._execute('config remove project name')
+        self.assertEqual(0, rv)
+        self.assertEqual(self.expected_results[test_name], output)
+        self.assertEqual('My Project', self.env.config.get('project', 'name'))
 
     # Permission tests
 
@@ -254,9 +337,8 @@ class TracadminTestCase(unittest.TestCase):
         error message.
         """
         test_name = sys._getframe().f_code.co_name
-        rv, output = self._execute('component add component1 new_user',
-                                   expect_exception=True)
-        self.assertEqual(-1, rv)
+        rv, output = self._execute('component add component1 new_user')
+        self.assertEqual(2, rv)
         self.assertEqual(self.expected_results[test_name], output)
 
     def test_component_rename_ok(self):
@@ -286,9 +368,8 @@ class TracadminTestCase(unittest.TestCase):
         test tries to rename a component to a name that already exists.
         """
         test_name = sys._getframe().f_code.co_name
-        rv, output = self._execute('component rename component1 component2',
-                                   expect_exception=True)
-        self.assertEqual(-1, rv)
+        rv, output = self._execute('component rename component1 component2')
+        self.assertEqual(2, rv)
         self.assertEqual(self.expected_results[test_name], output)
 
     def test_component_chown_ok(self):
@@ -311,7 +392,10 @@ class TracadminTestCase(unittest.TestCase):
         test_name = sys._getframe().f_code.co_name
         rv, output = self._execute('component chown bad_component changed_owner')
         self.assertEqual(2, rv)
-        self.assertEqual(self.expected_results[test_name], output)
+        # We currently trigger a deprecation warning with py26 so we 
+        # can currrently only verify that the end of the output string is
+        # correct
+        self.assertEqual(output.endswith(self.expected_results[test_name]), True)
 
     def test_component_remove_ok(self):
         """
@@ -365,9 +449,8 @@ class TracadminTestCase(unittest.TestCase):
         message.
         """
         test_name = sys._getframe().f_code.co_name
-        rv, output = self._execute('ticket_type add defect',
-                                   expect_exception=True)
-        self.assertEqual(-1, rv)
+        rv, output = self._execute('ticket_type add defect')
+        self.assertEqual(2, rv)
         self.assertEqual(self.expected_results[test_name], output)
 
     def test_ticket_type_change_ok(self):
@@ -397,9 +480,8 @@ class TracadminTestCase(unittest.TestCase):
         test tries to change a ticket type to another type that already exists.
         """
         test_name = sys._getframe().f_code.co_name
-        rv, output = self._execute('ticket_type change defect task',
-                                   expect_exception=True)
-        self.assertEqual(-1, rv)
+        rv, output = self._execute('ticket_type change defect task')
+        self.assertEqual(2, rv)
         self.assertEqual(self.expected_results[test_name], output)
 
     def test_ticket_type_remove_ok(self):
@@ -498,9 +580,8 @@ class TracadminTestCase(unittest.TestCase):
         error message.
         """
         test_name = sys._getframe().f_code.co_name
-        rv, output = self._execute('priority add blocker',
-                                   expect_exception=True)
-        self.assertEqual(-1, rv)
+        rv, output = self._execute('priority add blocker')
+        self.assertEqual(2, rv)
         self.assertEqual(self.expected_results[test_name], output)
 
     def test_priority_change_ok(self):
@@ -530,9 +611,8 @@ class TracadminTestCase(unittest.TestCase):
         test tries to change a priority to a name that already exists.
         """
         test_name = sys._getframe().f_code.co_name
-        rv, output = self._execute('priority change major minor',
-                                   expect_exception=True)
-        self.assertEqual(-1, rv)
+        rv, output = self._execute('priority change major minor')
+        self.assertEqual(2, rv)
         self.assertEqual(self.expected_results[test_name], output)
 
     def test_priority_remove_ok(self):
@@ -620,9 +700,8 @@ class TracadminTestCase(unittest.TestCase):
         """
         test_name = sys._getframe().f_code.co_name
         self._execute('severity add blocker')
-        rv, output = self._execute('severity add blocker',
-                                   expect_exception=True)
-        self.assertEqual(-1, rv)
+        rv, output = self._execute('severity add blocker')
+        self.assertEqual(2, rv)
         self.assertEqual(self.expected_results[test_name], output)
 
     def test_severity_change_ok(self):
@@ -655,9 +734,8 @@ class TracadminTestCase(unittest.TestCase):
         test_name = sys._getframe().f_code.co_name
         self._execute('severity add major')
         self._execute('severity add critical')
-        rv, output = self._execute('severity change critical major',
-                                   expect_exception=True)
-        self.assertEqual(-1, rv)
+        rv, output = self._execute('severity change critical major')
+        self.assertEqual(2, rv)
         self.assertEqual(self.expected_results[test_name], output)
 
     def test_severity_remove_ok(self):
@@ -748,9 +826,8 @@ class TracadminTestCase(unittest.TestCase):
         error message.
         """
         test_name = sys._getframe().f_code.co_name
-        rv, output = self._execute('version add 1.0 "%s"' % self._test_date,
-                                   expect_exception=True)
-        self.assertEqual(-1, rv)
+        rv, output = self._execute('version add 1.0 "%s"' % self._test_date)
+        self.assertEqual(2, rv)
         self.assertEqual(self.expected_results[test_name], output)
 
     def test_version_rename_ok(self):
@@ -850,7 +927,7 @@ class TracadminTestCase(unittest.TestCase):
         test_name = sys._getframe().f_code.co_name
         self._execute('milestone add new_milestone "%s"' % self._test_date)
         rv, output = self._execute('milestone list')
-        #self.assertEqual(0, rv)
+        self.assertEqual(0, rv)
         self.assertEqual(self.expected_results[test_name], output)
 
     def test_milestone_add_utf8_ok(self):
@@ -859,7 +936,7 @@ class TracadminTestCase(unittest.TestCase):
         test passes valid arguments and checks for success.
         """
         test_name = sys._getframe().f_code.co_name
-        self._execute(u'milestone add \xe9tat_final "%s"'  #\xc3\xa9
+        self._execute(u'milestone add \xa9tat_final "%s"'  #\xc2\xa9
                               % self._test_date)
         rv, output = self._execute('milestone list')
         self.assertEqual(0, rv)
@@ -873,9 +950,8 @@ class TracadminTestCase(unittest.TestCase):
         """
         test_name = sys._getframe().f_code.co_name
         rv, output = self._execute('milestone add milestone1 "%s"'
-                                   % self._test_date,
-                                   expect_exception=True)
-        self.assertEqual(-1, rv)
+                                   % self._test_date)
+        self.assertEqual(2, rv)
         self.assertEqual(self.expected_results[test_name], output)
 
     def test_milestone_rename_ok(self):
@@ -980,11 +1056,184 @@ class TracadminTestCase(unittest.TestCase):
 
     def test_backslash_use_ok(self):
         test_name = sys._getframe().f_code.co_name
-        self._execute('version add \\')
+        if self._admin.interactive:
+            self._execute('version add \\')
+        else:
+            self._execute(r"version add '\'")
         rv, output = self._execute('version list')
         self.assertEqual(0, rv)
         self.assertEqual(self.expected_results[test_name], output)
 
+    def test_session_list_no_sessions(self):
+        test_name = sys._getframe().f_code.co_name
+        rv, output = self._execute('session list authenticated')
+        self.assertEqual(0, rv)
+        self.assertEqual(self.expected_results[test_name], output)
+
+    def test_session_list_authenticated(self):
+        test_name = sys._getframe().f_code.co_name
+        _prep_session_table(self.env.get_db_cnx())
+        rv, output = self._execute('session list authenticated')
+        self.assertEqual(0, rv)
+        self.assertEqual(self.expected_results[test_name], output)
+
+    def test_session_list_anonymous(self):
+        test_name = sys._getframe().f_code.co_name
+        _prep_session_table(self.env.get_db_cnx())
+        rv, output = self._execute('session list anonymous')
+        self.assertEqual(0, rv)
+        self.assertEqual(self.expected_results[test_name], output)
+
+    def test_session_list_all(self):
+        test_name = sys._getframe().f_code.co_name
+        _prep_session_table(self.env.get_db_cnx())
+        if self._admin.interactive:
+            rv, output = self._execute("session list *")
+        else:
+            rv, output = self._execute("session list '*'")
+        self.assertEqual(0, rv)
+        self.assertEqual(self.expected_results[test_name], output)
+
+    def test_session_list_authenticated_sid(self):
+        test_name = sys._getframe().f_code.co_name
+        _prep_session_table(self.env.get_db_cnx())
+        rv, output = self._execute('session list name00')
+        self.assertEqual(0, rv)
+        self.assertEqual(self.expected_results[test_name], output)
+
+    def test_session_list_anonymous_sid(self):
+        test_name = sys._getframe().f_code.co_name
+        _prep_session_table(self.env.get_db_cnx())
+        rv, output = self._execute('session list name10:0')
+        self.assertEqual(0, rv)
+        self.assertEqual(self.expected_results[test_name], output)
+
+    def test_session_list_missing_sid(self):
+        test_name = sys._getframe().f_code.co_name
+        _prep_session_table(self.env.get_db_cnx())
+        rv, output = self._execute('session list thisdoesntexist')
+        self.assertEqual(0, rv)
+        self.assertEqual(self.expected_results[test_name], output)
+
+    def  test_session_add_missing_sid(self):
+        test_name = sys._getframe().f_code.co_name
+        rv, output = self._execute('session add')
+        self.assertEqual(2, rv)
+        self.assertEqual(self.expected_results[test_name], output)
+
+    def  test_session_add_duplicate_sid(self):
+        test_name = sys._getframe().f_code.co_name
+        _prep_session_table(self.env.get_db_cnx())
+        rv, output = self._execute('session add name00')
+        self.assertEqual(2, rv)
+        self.assertEqual(self.expected_results[test_name], output)
+
+    def  test_session_add_sid_all(self):
+        test_name = sys._getframe().f_code.co_name
+        rv, output = self._execute('session add john John john@example.org')
+        self.assertEqual(0, rv)
+        rv, output = self._execute('session list john')
+        self.assertEqual(self.expected_results[test_name]
+                         % {'today': format_date(None, console_date_format)},
+                         output)
+
+    def  test_session_add_sid(self):
+        test_name = sys._getframe().f_code.co_name
+        rv, output = self._execute('session add john')
+        self.assertEqual(0, rv)
+        rv, output = self._execute('session list john')
+        self.assertEqual(self.expected_results[test_name]
+                         % {'today': format_date(None, console_date_format)},
+                         output)
+
+    def  test_session_add_sid_name(self):
+        test_name = sys._getframe().f_code.co_name
+        rv, output = self._execute('session add john John')
+        self.assertEqual(0, rv)
+        rv, output = self._execute('session list john')
+        self.assertEqual(self.expected_results[test_name]
+                         % {'today': format_date(None, console_date_format)},
+                         output)
+
+    def  test_session_set_attr_name(self):
+        test_name = sys._getframe().f_code.co_name
+        _prep_session_table(self.env.get_db_cnx())
+        rv, output = self._execute('session set name name00 JOHN')
+        self.assertEqual(0, rv)
+        rv, output = self._execute('session list name00')
+        self.assertEqual(self.expected_results[test_name], output)
+
+    def  test_session_set_attr_email(self):
+        test_name = sys._getframe().f_code.co_name
+        _prep_session_table(self.env.get_db_cnx())
+        rv, output = self._execute('session set email name00 JOHN@EXAMPLE.ORG')
+        self.assertEqual(0, rv)
+        rv, output = self._execute('session list name00')
+        self.assertEqual(self.expected_results[test_name], output)
+
+    def  test_session_set_attr_missing_attr(self):
+        test_name = sys._getframe().f_code.co_name
+        rv, output = self._execute('session set')
+        self.assertEqual(2, rv)
+        self.assertEqual(self.expected_results[test_name], output)
+
+    def  test_session_set_attr_missing_value(self):
+        test_name = sys._getframe().f_code.co_name
+        rv, output = self._execute('session set name john')
+        self.assertEqual(2, rv)
+        self.assertEqual(self.expected_results[test_name], output)
+
+    def  test_session_set_attr_missing_sid(self):
+        test_name = sys._getframe().f_code.co_name
+        rv, output = self._execute('session set name')
+        self.assertEqual(2, rv)
+        self.assertEqual(self.expected_results[test_name], output)
+
+    def  test_session_set_attr_nonexistent_sid(self):
+        test_name = sys._getframe().f_code.co_name
+        rv, output = self._execute('session set name john foo')
+        self.assertEqual(2, rv)
+        self.assertEqual(self.expected_results[test_name], output)
+
+    def  test_session_delete_sid(self):
+        test_name = sys._getframe().f_code.co_name
+        _prep_session_table(self.env.get_db_cnx())
+        rv, output = self._execute('session delete name00')
+        self.assertEqual(0, rv)
+        rv, output = self._execute('session list nam00')
+        self.assertEqual(self.expected_results[test_name], output)
+
+    def  test_session_delete_missing_params(self):
+        test_name = sys._getframe().f_code.co_name
+        rv, output = self._execute('session delete')
+        self.assertEqual(0, rv)
+        self.assertEqual(self.expected_results[test_name], output)
+
+    def  test_session_delete_anonymous(self):
+        test_name = sys._getframe().f_code.co_name
+        _prep_session_table(self.env.get_db_cnx())
+        rv, output = self._execute('session delete anonymous')
+        self.assertEqual(0, rv)
+        rv, output = self._execute('session list *')
+        self.assertEqual(self.expected_results[test_name], output)
+
+    def test_session_delete_multiple_sids(self):
+        test_name = sys._getframe().f_code.co_name
+        _prep_session_table(self.env.get_db_cnx())
+        rv, output = self._execute('session delete name00 name01 name02 '
+                                   'name03')
+        self.assertEqual(0, rv)
+        rv, output = self._execute('session list *')
+        self.assertEqual(self.expected_results[test_name], output)
+
+    def  test_session_purge_age(self):
+        test_name = sys._getframe().f_code.co_name
+        _prep_session_table(self.env.get_db_cnx(), spread_visits=True)
+        rv, output = self._execute('session purge 20100112')
+        self.assertEqual(0, rv)
+        rv, output = self._execute('session list *')
+        self.assertEqual(self.expected_results[test_name], output)
+
 
 def suite():
     return unittest.makeSuite(TracadminTestCase, 'test')
