@@ -14,29 +14,40 @@
 #
 # Author: Jonas Borgström <jonas@edgewall.com>
 
-import os
-try:
-    import threading
-except ImportError:
-    import dummy_threading as threading
+import os.path
 import setuptools
 import sys
 from urlparse import urlsplit
 
 from trac import db_default
+from trac.admin import AdminCommandError, IAdminCommandProvider
+from trac.cache import CacheManager
 from trac.config import *
 from trac.core import Component, ComponentManager, implements, Interface, \
                       ExtensionPoint, TracError
-from trac.db import DatabaseManager
-from trac.util import create_file, get_pkginfo
-from trac.util.text import exception_to_unicode
-from trac.util.translation import _
+from trac.db.api import DatabaseManager, get_read_db, with_transaction
+from trac.util import copytree, create_file, get_pkginfo, makedirs
+from trac.util.compat import any
+from trac.util.concurrency import threading
+from trac.util.text import exception_to_unicode, path_to_unicode, printerr, \
+                           printout
+from trac.util.translation import _, N_
 from trac.versioncontrol import RepositoryManager
 from trac.web.href import Href
 
 __all__ = ['Environment', 'IEnvironmentSetupParticipant', 'open_environment']
 
 
+class ISystemInfoProvider(Interface):
+    """Provider of system information, displayed in the "About Trac" page and
+    in internal error reports.
+    """
+    def get_system_info():
+        """Yield a sequence of `(name, version)` tuples describing the name and
+        version information of external packages used by a component.
+        """
+
+
 class IEnvironmentSetupParticipant(Interface):
     """Extension point interface for components that need to participate in the
     creation and upgrading of Trac environments, for example to create
@@ -55,26 +66,35 @@ class IEnvironmentSetupParticipant(Interface):
     def upgrade_environment(db):
         """Actually perform an environment upgrade.
         
-        Implementations of this method should not commit any database
-        transactions. This is done implicitly after all participants have
-        performed the upgrades they need without an error being raised.
+        Implementations of this method don't need to commit any database
+        transactions. This is done implicitly for each participant
+        if the upgrade succeeds without an error being raised.
+
+        However, if the `upgrade_environment` consists of small, restartable,
+        steps of upgrade, it can decide to commit on its own after each
+        successful step.
         """
 
 
 class Environment(Component, ComponentManager):
-    """Trac stores project information in a Trac environment.
+    """Trac environment manager.
 
-    A Trac environment consists of a directory structure containing among other
-    things:
-     * a configuration file.
+    Trac stores project information in a Trac environment. It consists of a
+    directory structure containing among other things:
+     * a configuration file
      * an SQLite database (stores tickets, wiki pages...)
-     * Project specific templates and plugins.
-     * wiki and ticket attachments.
-    """   
+     * project-specific templates and plugins
+     * wiki and ticket attachments
+    """
+    implements(ISystemInfoProvider)
+
+    required = True
+    
+    system_info_providers = ExtensionPoint(ISystemInfoProvider)
     setup_participants = ExtensionPoint(IEnvironmentSetupParticipant)
 
     shared_plugins_dir = PathOption('inherit', 'plugins_dir', '',
-        """Path of the directory containing additional plugins.
+        """Path to the //shared plugins directory//.
         
         Plugins in that directory are loaded in addition to those in the
         directory of the environment `plugins`, with this one taking 
@@ -116,7 +136,7 @@ class Environment(Component, ComponentManager):
 
     project_url = Option('project', 'url', '',
         """URL of the main project web site, usually the website in which
-        the `base_url` resides.""")
+        the `base_url` resides. This is used in notification e-mails.""")
 
     project_admin = Option('project', 'admin', '',
         """E-Mail address of the project's administrator.""")
@@ -130,9 +150,9 @@ class Environment(Component, ComponentManager):
         (''since 0.11.3'')""")
 
     project_footer = Option('project', 'footer',
-                            'Visit the Trac open source project at<br />'
-                            '<a href="http://trac.edgewall.org/">'
-                            'http://trac.edgewall.org/</a>',
+                            N_('Visit the Trac open source project at<br />'
+                               '<a href="http://trac.edgewall.org/">'
+                               'http://trac.edgewall.org/</a>'),
         """Page footer text (right-aligned).""")
 
     project_icon = Option('project', 'icon', 'common/trac.ico',
@@ -144,7 +164,9 @@ class Environment(Component, ComponentManager):
         Should be one of (`none`, `file`, `stderr`, `syslog`, `winlog`).""")
 
     log_file = Option('logging', 'log_file', 'trac.log',
-        """If `log_type` is `file`, this should be a path to the log-file.""")
+        """If `log_type` is `file`, this should be a path to the log-file.
+        Relative paths are resolved relative to the `log` directory of the
+        environment.""")
 
     log_level = Option('logging', 'log_level', 'DEBUG',
         """Level of verbosity in log.
@@ -159,7 +181,7 @@ class Environment(Component, ComponentManager):
         Trac[$(module)s] $(levelname)s: $(message)s
 
         In addition to regular key names supported by the Python logger library
-        library (see http://docs.python.org/lib/node422.html), one could use:
+        (see http://docs.python.org/library/logging.html), one could use:
          - $(path)s     the path for the current environment
          - $(basename)s the last path component of the current environment
          - $(project)s  the project name
@@ -185,33 +207,40 @@ class Environment(Component, ComponentManager):
         ComponentManager.__init__(self)
 
         self.path = path
-        self.setup_config(load_defaults=create)
-        self.setup_log()
-
-        from trac import core, __version__ as VERSION
-        trac_version = get_pkginfo(core).get('version', VERSION)
-        self.systeminfo = [
-            ('Trac', trac_version),
-            ('Python', sys.version),
-            ('setuptools', setuptools.__version__),
-            ]
-        self.log.info('-' * 32 + ' environment startup [Trac %s] ' + '-' * 32,
-                      trac_version)
+        self.systeminfo = []
         self._href = self._abs_href = None
 
-        from trac.loader import load_components
-        plugins_dir = self.shared_plugins_dir
-        load_components(self, plugins_dir and (plugins_dir,))
-
         if create:
             self.create(options)
         else:
             self.verify()
+            self.setup_config()
 
         if create:
             for setup_participant in self.setup_participants:
                 setup_participant.environment_created()
 
+    def get_systeminfo(self):
+        """Return a list of `(name, version)` tuples describing the name and
+        version information of external packages used by Trac and plugins.
+        """
+        info = self.systeminfo[:]
+        for provider in self.system_info_providers:
+            info.extend(provider.get_system_info() or [])
+        info.sort(key=lambda (name, version): (name != 'Trac', name.lower()))
+        return info
+
+    # ISystemInfoProvider methods
+
+    def get_system_info(self):
+        from trac import core, __version__ as VERSION
+        yield 'Trac', get_pkginfo(core).get('version', VERSION)
+        yield 'Python', sys.version
+        yield 'setuptools', setuptools.__version__
+        from trac.util.datefmt import pytz
+        if pytz is not None:
+            yield 'pytz', pytz.__version__
+    
     def component_activated(self, component):
         """Initialize additional member variables for components.
         
@@ -222,24 +251,35 @@ class Environment(Component, ComponentManager):
         component.config = self.config
         component.log = self.log
 
-    def is_component_enabled(self, cls):
-        """Implemented to only allow activation of components that are not
-        disabled in the configuration.
-        
-        This is called by the `ComponentManager` base class when a component is
-        about to be activated. If this method returns false, the component does
-        not get activated."""
-        if not hasattr(self, '_rules'):
+    def _component_name(self, name_or_class):
+        name = name_or_class
+        if not isinstance(name_or_class, basestring):
+            name = name_or_class.__module__ + '.' + name_or_class.__name__
+        return name.lower()
+
+    @property
+    def _component_rules(self):
+        try:
+            return self._rules
+        except AttributeError:
             self._rules = {}
             for name, value in self.config.options('components'):
                 if name.endswith('.*'):
                     name = name[:-2]
                 self._rules[name.lower()] = value.lower() in ('enabled', 'on')
-
-        if not isinstance(cls, basestring):
-            component_name = (cls.__module__ + '.' + cls.__name__).lower()
-        else:
-            component_name = cls.lower()
+            return self._rules
+        
+    def is_component_enabled(self, cls):
+        """Implemented to only allow activation of components that are not
+        disabled in the configuration.
+        
+        This is called by the `ComponentManager` base class when a component is
+        about to be activated. If this method returns `False`, the component
+        does not get activated. If it returns `None`, the component only gets
+        activated if it is located in the `plugins` directory of the
+        enironment.
+        """
+        component_name = self._component_name(cls)
 
         # Disable the pre-0.11 WebAdmin plugin
         # Please note that there's no recommendation to uninstall the
@@ -253,9 +293,10 @@ class Environment(Component, ComponentManager):
                           'instead.')
             return False
         
+        rules = self._component_rules
         cname = component_name
         while cname:
-            enabled = self._rules.get(cname)
+            enabled = rules.get(cname)
             if enabled is not None:
                 return enabled
             idx = cname.rfind('.')
@@ -263,13 +304,12 @@ class Environment(Component, ComponentManager):
                 break
             cname = cname[:idx]
 
-        # versioncontrol components are enabled if the repository is configured
-        # FIXME: this shouldn't be hardcoded like this
-        if component_name.startswith('trac.versioncontrol.'):
-            return self.config.get('trac', 'repository_dir') != ''
-
         # By default, all components in the trac package are enabled
-        return component_name.startswith('trac.')
+        return component_name.startswith('trac.') or None
+
+    def enable_component(self, cls):
+        """Enable a component or module."""
+        self._component_rules[self._component_name(cls)] = True
 
     def verify(self):
         """Verify that the provided path points to a valid Trac environment
@@ -281,28 +321,48 @@ class Environment(Component, ComponentManager):
             fd.close()
 
     def get_db_cnx(self):
-        """Return a database connection from the connection pool."""
-        return DatabaseManager(self).get_connection()
+        """Return a database connection from the connection pool (deprecated)
 
-    def shutdown(self, tid=None, except_logging=False):
+        Use `with_transaction` for obtaining a writable database connection
+        and `get_read_db` for anything else.
+        """
+        return get_read_db(self)
+
+    def with_transaction(self, db=None):
+        """Decorator for transaction functions.
+
+        See `trac.db.api.with_transaction` for detailed documentation."""
+        return with_transaction(self, db)
+
+    def get_read_db(self):
+        """Return a database connection for read purposes.
+
+        See `trac.db.api.get_read_db` for detailed documentation."""
+        return get_read_db(self)
+
+    def shutdown(self, tid=None):
         """Close the environment."""
         RepositoryManager(self).shutdown(tid)
         DatabaseManager(self).shutdown(tid)
-        if tid is None and not except_logging and \
-                hasattr(self.log, '_trac_handler'):
-            hdlr = self.log._trac_handler
-            self.log.removeHandler(hdlr)
-            hdlr.flush()
-            hdlr.close()
-            del self.log._trac_handler
+        if tid is None:
+            self.log.removeHandler(self._log_handler)
+            self._log_handler.flush()
+            self._log_handler.close()
+            del self._log_handler
 
-    def get_repository(self, authname=None):
-        """Return the version control repository configured for this
-        environment.
+    def get_repository(self, reponame=None, authname=None):
+        """Return the version control repository with the given name, or the
+        default repository if `None`.
         
-        @param authname: user name for authorization
+        The standard way of retrieving repositories is to use the methods
+        of `RepositoryManager`. This method is retained here for backward
+        compatibility.
+        
+        @param reponame: the name of the repository
+        @param authname: the user name for authorization (not used anymore,
+                         left here for compatibility with 0.11)
         """
-        return RepositoryManager(self).get_repository(authname)
+        return RepositoryManager(self).get_repository(reponame)
 
     def create(self, options=[]):
         """Create the basic directory structure of the environment, initialize
@@ -327,15 +387,16 @@ class Environment(Component, ComponentManager):
 
         # Setup the default configuration
         os.mkdir(os.path.join(self.path, 'conf'))
-        create_file(os.path.join(self.path, 'conf', 'trac.ini'))
         create_file(os.path.join(self.path, 'conf', 'trac.ini.sample'))
-        skip_defaults = options and ('inherit', 'file') in [(section, option) \
-                for (section, option, value) in options]
-        self.setup_config(load_defaults=not skip_defaults)
+        config = Configuration(os.path.join(self.path, 'conf', 'trac.ini'))
         for section, name, value in options:
-            self.config.set(section, name, value)
-        self.config.save()
-        self.config.parse_if_needed() # Full reload to get 'inherit' working
+            config.set(section, name, value)
+        config.save()
+        self.setup_config()
+        if not any((section, option) == ('inherit', 'file')
+                   for section, option, value in options):
+            self.config.set_defaults(self)
+            self.config.save()
 
         # Create the database
         DatabaseManager(self).init_db()
@@ -359,15 +420,14 @@ class Environment(Component, ComponentManager):
         row = cursor.fetchone()
         return row and int(row[0])
 
-    def setup_config(self, load_defaults=False):
+    def setup_config(self):
         """Load the configuration file."""
-        self.config = Configuration(os.path.join(self.path, 'conf', 'trac.ini'))
-        if load_defaults:
-            for section, default_options in self.config.defaults().items():
-                for name, value in default_options.items():
-                    if self.config.parent and name in self.config.parent[section]:
-                        value = None
-                    self.config.set(section, name, value)
+        self.config = Configuration(os.path.join(self.path, 'conf',
+                                                 'trac.ini'))
+        self.setup_log()
+        from trac.loader import load_components
+        plugins_dir = self.shared_plugins_dir
+        load_components(self, plugins_dir and (plugins_dir,))
 
     def get_templates_dir(self):
         """Return absolute path to the templates directory."""
@@ -383,7 +443,7 @@ class Environment(Component, ComponentManager):
 
     def setup_log(self):
         """Initialize the logging sub-system."""
-        from trac.log import logger_factory
+        from trac.log import logger_handler_factory
         logtype = self.log_type
         logfile = self.log_file
         if logtype == 'file' and not os.path.isabs(logfile):
@@ -394,8 +454,11 @@ class Environment(Component, ComponentManager):
                      .replace('%(path)s', self.path) \
                      .replace('%(basename)s', os.path.basename(self.path)) \
                      .replace('%(project)s', self.project_name)
-        self.log = logger_factory(logtype, logfile, self.log_level, self.path,
-                                  format=format)
+        self.log, self._log_handler = logger_handler_factory(
+            logtype, logfile, self.log_level, self.path, format=format)
+        from trac import core, __version__ as VERSION
+        self.log.info('-' * 32 + ' environment startup [Trac %s] ' + '-' * 32,
+                      get_pkginfo(core).get('version', VERSION))
 
     def get_known_users(self, cnx=None):
         """Generator that yields information about all known users, i.e. users
@@ -418,11 +481,11 @@ class Environment(Component, ComponentManager):
                        " LEFT JOIN session_attribute AS e ON (e.sid=s.sid "
                        "  AND e.authenticated=1 AND e.name = 'email') "
                        "WHERE s.authenticated=1 ORDER BY s.sid")
-        for username,name,email in cursor:
+        for username, name, email in cursor:
             yield username, name, email
 
     def backup(self, dest=None):
-        """Simple SQLite-specific backup of the database.
+        """Create a backup of the database.
 
         @param dest: Destination file; if not specified, the backup is stored in
                      a file called db_name.trac_version.bak
@@ -442,40 +505,39 @@ class Environment(Component, ComponentManager):
     def upgrade(self, backup=False, backup_dest=None):
         """Upgrade database.
         
-        Each db version should have its own upgrade module, names
-        upgrades/dbN.py, where 'N' is the version number (int).
-
         @param backup: whether or not to backup before upgrading
         @param backup_dest: name of the backup file
         @return: whether the upgrade was performed
         """
-        db = self.get_db_cnx()
-
         upgraders = []
+        db = self.get_read_db()
         for participant in self.setup_participants:
             if participant.environment_needs_upgrade(db):
                 upgraders.append(participant)
         if not upgraders:
-            return False
+            return
 
         if backup:
             self.backup(backup_dest)
+
         for participant in upgraders:
-            participant.upgrade_environment(db)
-        db.commit()
-
-        # Database schema may have changed, so close all connections
-        self.shutdown(except_logging=True)
-
+            self.log.info("%s.%s upgrading...", participant.__module__,
+                          participant.__class__.__name__)
+            with_transaction(self)(participant.upgrade_environment)
+            # Database schema may have changed, so close all connections
+            DatabaseManager(self).shutdown()
         return True
 
-    def _get_href(self):
+    @property
+    def href(self):
+        """The application root path"""
         if not self._href:
             self._href = Href(urlsplit(self.abs_href.base)[2])
         return self._href
-    href = property(_get_href, 'The application root path')
 
-    def _get_abs_href(self):
+    @property
+    def abs_href(self):
+        """The application URL"""
         if not self._abs_href:
             if not self.base_url:
                 self.log.warn('base_url option not set in configuration, '
@@ -484,23 +546,27 @@ class Environment(Component, ComponentManager):
             else:
                 self._abs_href = Href(self.base_url)
         return self._abs_href
-    abs_href = property(_get_abs_href, 'The application URL')
 
 
 class EnvironmentSetup(Component):
+    """Manage automatic environment upgrades."""
+    
+    required = True
+
     implements(IEnvironmentSetupParticipant)
 
     # IEnvironmentSetupParticipant methods
 
     def environment_created(self):
         """Insert default data into the database."""
-        db = self.env.get_db_cnx()
-        cursor = db.cursor()
-        for table, cols, vals in db_default.get_data(db):
-            cursor.executemany("INSERT INTO %s (%s) VALUES (%s)" % (table,
-                               ','.join(cols), ','.join(['%s' for c in cols])),
-                               vals)
-        db.commit()
+        @self.env.with_transaction()
+        def do_db_populate(db):
+            cursor = db.cursor()
+            for table, cols, vals in db_default.get_data(db):
+                cursor.executemany("INSERT INTO %s (%s) VALUES (%s)"
+                                   % (table, ','.join(cols),
+                                      ','.join(['%s' for c in cols])),
+                                   vals)
         self._update_sample_config()
 
     def environment_needs_upgrade(self, db):
@@ -514,6 +580,9 @@ class EnvironmentSetup(Component):
         return True
 
     def upgrade_environment(self, db):
+        """Each db version should have its own upgrade module, named
+        upgrades/dbN.py, where 'N' is the version number (int).
+        """
         cursor = db.cursor()
         dbver = self.env.get_version()
         for i in range(dbver + 1, db_default.db_version + 1):
@@ -525,10 +594,11 @@ class EnvironmentSetup(Component):
                 raise TracError(_('No upgrade module for version %(num)i '
                                   '(%(version)s.py)', num=i, version=name))
             script.do_upgrade(self.env, i, cursor)
-        cursor.execute("UPDATE system SET value=%s WHERE "
-                       "name='database_version'", (db_default.db_version,))
-        self.log.info('Upgraded database version from %d to %d',
-                      dbver, db_default.db_version)
+            cursor.execute("""
+                UPDATE system SET value=%s WHERE name='database_version'
+                """, (i,))
+            self.log.info('Upgraded database version from %d to %d', i - 1, i)
+            db.commit()
         self._update_sample_config()
 
     # Internal methods
@@ -564,8 +634,6 @@ def open_environment(env_path=None, use_cache=False):
                       invocations of this function
     @return: the `Environment` object
     """
-    global env_cache, env_cache_lock
-
     if not env_path:
         env_path = os.getenv('TRAC_ENV')
     if not env_path:
@@ -588,6 +656,8 @@ def open_environment(env_path=None, use_cache=False):
                 env = None
             if env is None:
                 env = env_cache.setdefault(env_path, open_environment(env_path))
+            else:
+                CacheManager(env).reset_metadata()
         finally:
             env_cache_lock.release()
     else:
@@ -604,3 +674,143 @@ def open_environment(env_path=None, use_cache=False):
                               path=env_path))
 
     return env
+
+
+class EnvironmentAdmin(Component):
+    """trac-admin command provider for environment administration."""
+    
+    implements(IAdminCommandProvider)
+    
+    # IAdminCommandProvider methods
+    
+    def get_admin_commands(self):
+        yield ('deploy', '<directory>',
+               'Extract static resources from Trac and all plugins',
+               None, self._do_deploy)
+        yield ('hotcopy', '<backupdir>',
+               'Make a hot backup copy of an environment',
+               None, self._do_hotcopy)
+        yield ('upgrade', '',
+               'Upgrade database to current version',
+               None, self._do_upgrade)
+    
+    def _do_deploy(self, dest):
+        target = os.path.normpath(dest)
+        chrome_target = os.path.join(target, 'htdocs')
+        script_target = os.path.join(target, 'cgi-bin')
+
+        # Copy static content
+        makedirs(target, overwrite=True)
+        makedirs(chrome_target, overwrite=True)
+        from trac.web.chrome import Chrome
+        printout(_("Copying resources from:"))
+        for provider in Chrome(self.env).template_providers:
+            paths = list(provider.get_htdocs_dirs() or [])
+            if not len(paths):
+                continue
+            printout('  %s.%s' % (provider.__module__, 
+                                  provider.__class__.__name__))
+            for key, root in paths:
+                source = os.path.normpath(root)
+                printout('   ', source)
+                if os.path.exists(source):
+                    dest = os.path.join(chrome_target, key)
+                    copytree(source, dest, overwrite=True)
+
+        # Create and copy scripts
+        makedirs(script_target, overwrite=True)
+        printout(_("Creating scripts."))
+        data = {'env': self.env, 'executable': sys.executable}
+        for script in ('cgi', 'fcgi', 'wsgi'):
+            dest = os.path.join(script_target, 'trac.' + script)
+            template = Chrome(self.env).load_template('deploy_trac.' + script,
+                                                      'text')
+            stream = template.generate(**data)
+            out = file(dest, 'w')
+            try:
+                stream.render('text', out=out, encoding='utf-8')
+            finally:
+                out.close()
+    
+    def _do_hotcopy(self, dest):
+        if os.path.exists(dest):
+            raise TracError(_("hotcopy can't overwrite existing '%(dest)s'",
+                              dest=path_to_unicode(dest)))
+        import shutil
+
+        # Bogus statement to lock the database while copying files
+        cnx = self.env.get_db_cnx()
+        cursor = cnx.cursor()
+        cursor.execute("UPDATE system SET name=NULL WHERE name IS NULL")
+
+        try:
+            printout(_('Hotcopying %(src)s to %(dst)s ...', 
+                       src=path_to_unicode(self.env.path),
+                       dst=path_to_unicode(dest)))
+            db_str = self.env.config.get('trac', 'database')
+            prefix, db_path = db_str.split(':', 1)
+            if prefix == 'sqlite':
+                # don't copy the journal (also, this would fail on Windows)
+                db = os.path.join(self.env.path, os.path.normpath(db_path))
+                skip = [db + '-journal', db + '-stmtjrnl']
+            else:
+                skip = []
+            try:
+                copytree(self.env.path, dest, symlinks=1, skip=skip)
+                retval = 0
+            except shutil.Error, e:
+                retval = 1
+                printerr(_('The following errors happened while copying '
+                           'the environment:'))
+                for (src, dst, err) in e.args[0]:
+                    if src in err:
+                        printerr('  %s' % err)
+                    else:
+                        printerr("  %s: '%s'" % (err, path_to_unicode(src)))
+        finally:
+            # Unlock database
+            cnx.rollback()
+
+        printout(_("Hotcopy done."))
+        return retval
+    
+    def _do_upgrade(self, no_backup=None):
+        if no_backup not in (None, '-b', '--no-backup'):
+            raise AdminCommandError(_("Invalid arguments"), show_usage=True)
+        
+        if not self.env.needs_upgrade():
+            printout(_("Database is up to date, no upgrade necessary."))
+            return
+
+        try:
+            self.env.upgrade(backup=no_backup is None)
+        except TracError, e:
+            raise TracError(_("Backup failed: %(msg)s.\nUse '--no-backup' to "
+                              "upgrade without doing a backup.",
+                              msg=unicode(e)))
+
+        # Remove wiki-macros if it is empty and warn if it isn't
+        wiki_macros = os.path.join(self.env.path, 'wiki-macros')
+        try:
+            entries = os.listdir(wiki_macros)
+        except OSError:
+            pass
+        else:
+            if entries:
+                printerr(_("Warning: the wiki-macros directory in the "
+                           "environment is non-empty, but Trac\n"
+                           "doesn't load plugins from there anymore. "
+                           "Please remove it by hand."))
+            else:
+                try:
+                    os.rmdir(wiki_macros)
+                except OSError, e:
+                    printerr(_("Error while removing wiki-macros: %(err)s\n"
+                               "Trac doesn't load plugins from wiki-macros "
+                               "anymore. Please remove it by hand.",
+                               err=exception_to_unicode(e)))
+        
+        printout(_("Upgrade done.\n\n"
+                   "You may want to upgrade the Trac documentation now by "
+                   "running:\n\n  trac-admin %(path)s wiki upgrade",
+                   path=path_to_unicode(self.env.path)))
