@@ -34,7 +34,7 @@ from trac.util import Ranges, as_bool
 from trac.util.datefmt import format_datetime, from_utimestamp, parse_date, \
                               to_timestamp, to_utimestamp, utc
 from trac.util.presentation import Paginator
-from trac.util.text import empty, shorten_line
+from trac.util.text import empty, shorten_line, quote_query_string
 from trac.util.translation import _, tag_
 from trac.web import arg_list_to_args, parse_arg_list, IRequestHandler
 from trac.web.href import Href
@@ -129,8 +129,11 @@ class Query(object):
 
         constraint_cols = {}
         for clause in self.constraints:
-            for k, v in clause.iteritems():
-                constraint_cols.setdefault(k, []).append(v)
+            for k, v in clause.items():
+                if k == 'id' or k in field_names:
+                    constraint_cols.setdefault(k, []).append(v)
+                else:
+                    clause.pop(k)
         self.constraint_cols = constraint_cols
 
     _clause_splitter = re.compile(r'(?<!\\)&')
@@ -701,6 +704,8 @@ class Query(object):
                     if val[:1] in ('~', '^', '$') \
                                         and not val in self.substitutions:
                         mode, val = val[:1], val[1:]
+                    if req:
+                        val = val.replace('$USER', req.authname)
                     constraint['mode'] = (neg and '!' or '') + mode
                     constraint['values'].append(val)
                 constraints[k] = constraint
@@ -883,17 +888,25 @@ class QueryModule(Component):
                 args = arg_list_to_args(arg_list)
                 constraints = self._get_constraints(arg_list=arg_list)
             else:
-                constraints = Query.from_string(self.env, qstring).constraints
-                # Substitute $USER, or ensure no field constraints that depend
-                # on $USER are used if we have no username.
-                for clause in constraints:
-                    for field, vals in clause.items():
-                        for (i, val) in enumerate(vals):
-                            if user:
-                                vals[i] = val.replace('$USER', user)
-                            elif val.endswith('$USER'):
-                                del clause[field]
-                                break
+                query = Query.from_string(self.env, qstring)
+                args = {'order': query.order, 'group': query.group,
+                        'col': query.cols, 'max': query.max}
+                if query.desc:
+                    args['desc'] = '1'
+                if query.groupdesc:
+                    args['groupdesc'] = '1'
+                constraints = query.constraints
+
+            # Substitute $USER, or ensure no field constraints that depend
+            # on $USER are used if we have no username.
+            for clause in constraints:
+                for field, vals in clause.items():
+                    for (i, val) in enumerate(vals):
+                        if user:
+                            vals[i] = val.replace('$USER', user)
+                        elif val.endswith('$USER'):
+                            del clause[field]
+                            break
 
         cols = args.get('col')
         if isinstance(cols, basestring):
@@ -1152,8 +1165,9 @@ class QueryModule(Component):
 
     def _format_link(self, formatter, ns, query, label):
         if query.startswith('?'):
+            query = quote_query_string(query)
             return tag.a(label, class_='query',
-                         href=formatter.href.query() + query.replace(' ', '+'))
+                         href=formatter.href.query() + query)
         else:
             try:
                 query = Query.from_string(self.env, query)
@@ -1281,7 +1295,8 @@ class TicketQueryMacro(WikiMacroBase):
         tickets = query.execute(req)
 
         if format == 'table':
-            data = query.template_data(formatter.context, tickets)
+            data = query.template_data(formatter.context, tickets,
+                                       req=formatter.context.req)
 
             add_stylesheet(req, 'common/css/report.css')
             
