@@ -455,6 +455,7 @@ class TicketCommentEditTestCase(TicketCommentTestCase):
             comment=dict(author='jack', old='1', new='New comment 1'),
             _comment0=dict(author='joe', old='Comment 1',
                            new=str(to_utimestamp(t))))
+        self.assertEqual(t, Ticket(self.env, self.id)['changetime'])
 
     def test_threading(self):
         """Check modification of a "threaded" comment"""
@@ -503,7 +504,7 @@ class TicketCommentEditTestCase(TicketCommentTestCase):
                            new=str(to_utimestamp(t))))
         
     def test_modify_missing_cnums_and_comment(self):
-        """Editing a comments when all cnums are missing and one comment
+        """Editing a comment when all cnums are missing and one comment
         field is missing
         """
         cursor = self.db.cursor()
@@ -658,6 +659,15 @@ class TicketCommentDeleteTestCase(TicketCommentTestCase):
             keywords=dict(author='joe', old='1, 2', new='a'),
             foo=dict(author='joe', old='change2', new='change4'))
         
+    def test_delete_all_comments(self):
+        # See ticket:10338
+        ticket = Ticket(self.env, self.id)
+        ticket.delete_change(4)
+        ticket.delete_change(3)
+        ticket.delete_change(2)
+        ticket.delete_change(1)
+        self.assertEquals(ticket['time'], ticket['changetime'])
+
 
 class EnumTestCase(unittest.TestCase):
 
