@@ -142,7 +142,7 @@ def format_datetime(t=None, format='%x %X', tzinfo=None):
         else:
             format = '%Y-%m-%dT%H:%M:%S%z'
             normalize_Z = True
-    text = t.strftime(format)
+    text = t.strftime(str(format))
     if normalize_Z:
         text = text.replace('+0000', 'Z')
         if not text.endswith('Z'):
@@ -360,6 +360,11 @@ class FixedOffset(tzinfo):
             raise ValueError('Not naive datetime (tzinfo is already set)')
         return dt.replace(tzinfo=self)
 
+    def normalize(self, dt, is_dst=False):
+        if dt.tzinfo is None:
+            raise ValueError('Naive time (no tzinfo set)')
+        return dt
+
 
 STDOFFSET = timedelta(seconds=-time.timezone)
 if time.daylight:
@@ -412,6 +417,11 @@ class LocalTimezone(tzinfo):
             raise ValueError('Not naive datetime (tzinfo is already set)')
         return dt.replace(tzinfo=self)
 
+    def normalize(self, dt, is_dst=False):
+        if dt.tzinfo is None:
+            raise ValueError('Naive time (no tzinfo set)')
+        return dt
+
 
 utc = FixedOffset(0, 'UTC')
 utcmin = datetime.min.replace(tzinfo=utc)
