@@ -17,6 +17,7 @@ tasks such as grouping or pagination.
 """
 
 from math import ceil
+import re
 
 __all__ = ['classes', 'first_last', 'group', 'istext', 'prepared_paginate', 
            'paginate', 'Paginator']
@@ -169,6 +170,7 @@ def paginate(items, page=0, max_per_page=10):
         retval = items[start:stop]
     except TypeError: # Slicing not supported, so iterate through the whole list
         retval = []
+        idx = -1 # Needed if items = []
         for idx, item in enumerate(items):
             if start <= idx < stop:
                 retval.append(item)
@@ -215,17 +217,17 @@ class Paginator(object):
     def __setitem__(self, idx, value):
         self.items[idx] = value
 
+    @property
     def has_more_pages(self):
         return self.num_pages > 1
-    has_more_pages = property(has_more_pages)
 
+    @property
     def has_next_page(self):
         return self.page + 1 < self.num_pages
-    has_next_page = property(has_next_page)
 
+    @property
     def has_previous_page(self):
         return self.page > 0
-    has_previous_page = property(has_previous_page)
    
     def get_shown_pages(self, page_index_count = 11):
         if self.has_more_pages == False:
@@ -276,3 +278,42 @@ def separated(items, sep=','):
         yield last, sep
         last = i
     yield last, None
+
+
+try:
+    from json import dumps
+
+    _js_quote = dict((c, '\\u%04x' % ord(c)) for c in '&<>')
+    _js_quote_re = re.compile('[' + ''.join(_js_quote) + ']')
+    
+    def to_json(value):
+        """Encode `value` to JSON."""
+        def replace(match):
+            return _js_quote[match.group(0)]
+        text = dumps(value, sort_keys=True, separators=(',', ':'))
+        return _js_quote_re.sub(replace, text)
+
+except ImportError:
+    from trac.util.text import javascript_quote
+    
+    def to_json(value):
+        """Encode `value` to JSON."""
+        if isinstance(value, basestring):
+            return '"%s"' % javascript_quote(value)
+        elif value is None:
+            return 'null'
+        elif value is False:
+            return 'false'
+        elif value is True:
+            return 'true'
+        elif isinstance(value, (int, long)):
+            return str(value)
+        elif isinstance(value, float):
+            return repr(value)
+        elif isinstance(value, (list, tuple)):
+            return '[%s]' % ','.join(to_json(each) for each in value)
+        elif isinstance(value, dict):
+            return '{%s}' % ','.join('%s:%s' % (to_json(k), to_json(v))
+                                     for k, v in sorted(value.iteritems()))
+        else:
+            raise TypeError('Cannot encode type %s' % value.__class__.__name__)
