@@ -16,11 +16,9 @@
 # Author: Jonas Borgström <jonas@edgewall.com>
 #         Christopher Lenz <cmlenz@gmx.de>
 
-from datetime import datetime
 import pkg_resources
 import re
 
-from genshi.core import Markup
 from genshi.builder import tag
 
 from trac.attachment import AttachmentModule
@@ -32,16 +30,16 @@ from trac.resource import *
 from trac.search import ISearchSource, search_to_sql, shorten_result
 from trac.timeline.api import ITimelineEventProvider
 from trac.util import get_reporter_id
-from trac.util.datefmt import to_timestamp, utc
+from trac.util.datefmt import from_utimestamp, to_utimestamp
 from trac.util.text import shorten_line
-from trac.util.translation import _
+from trac.util.translation import _, tag_
 from trac.versioncontrol.diff import get_diff_options, diff_blocks
 from trac.web.chrome import add_ctxtnav, add_link, add_notice, add_script, \
                             add_stylesheet, add_warning, prevnext_nav, \
-                            INavigationContributor, ITemplateProvider
-from trac.web import IRequestHandler
-from trac.wiki.api import IWikiPageManipulator, WikiSystem
-from trac.wiki.formatter import format_to
+                            Chrome, INavigationContributor, ITemplateProvider
+from trac.web.api import IRequestHandler
+from trac.wiki.api import IWikiPageManipulator, WikiSystem, validate_page_name
+from trac.wiki.formatter import format_to, OneLinerFormatter
 from trac.wiki.model import WikiPage
  
 class InvalidWikiPage(TracError):
@@ -71,11 +69,6 @@ class WikiModule(Component):
                9)
 
     def convert_content(self, req, mimetype, content, key):
-        # Tell the browser that the content should be downloaded and
-        # not rendered. The x=y part is needed to keep Safari from being 
-        # confused by the multiple content-disposition headers.
-        req.send_header('Content-Disposition', 'attachment; x=y')
-
         return (content, 'text/plain;charset=utf-8')
 
     # INavigationContributor methods
@@ -94,7 +87,8 @@ class WikiModule(Component):
     # IPermissionRequestor methods
 
     def get_permission_actions(self):
-        actions = ['WIKI_CREATE', 'WIKI_DELETE', 'WIKI_MODIFY', 'WIKI_VIEW']
+        actions = ['WIKI_CREATE', 'WIKI_DELETE', 'WIKI_MODIFY', 'WIKI_RENAME',
+                   'WIKI_VIEW']
         return actions + [('WIKI_ADMIN', actions)]
 
     # IRequestHandler methods
@@ -112,8 +106,13 @@ class WikiModule(Component):
         version = req.args.get('version')
         old_version = req.args.get('old_version')
 
-        if pagename.endswith('/'):
-            req.redirect(req.href.wiki(pagename.strip('/')))
+        if pagename.startswith('/') or pagename.endswith('/') or \
+                '//' in pagename:
+            pagename = re.sub(r'/{2,}', '/', pagename.strip('/'))
+            req.redirect(req.href.wiki(pagename))
+        if not validate_page_name(pagename):
+            raise TracError(_("Invalid Wiki page name '%(name)s'",
+                              name=pagename))
 
         page = WikiPage(self.env, pagename)
         versioned_page = WikiPage(self.env, pagename, version=version)
@@ -121,10 +120,10 @@ class WikiModule(Component):
         req.perm(page.resource).require('WIKI_VIEW')
         req.perm(versioned_page.resource).require('WIKI_VIEW')
 
-        if version and versioned_page.version == 0 and \
-               page.version != 0:
-            raise TracError(_('No version "%(num)s" for Wiki page "%(name)s"',
-                              num=version, name=page.name))
+        if version and versioned_page.version != int(version):
+            raise ResourceNotFound(
+                _('No version "%(num)s" for Wiki page "%(name)s"',
+                  num=version, name=page.name))
 
         add_stylesheet(req, 'common/css/wiki.css')
 
@@ -138,6 +137,7 @@ class WikiModule(Component):
                     if a in req.args:
                         action = a
                         break
+                versioned_page.text = req.args.get('text')
                 valid = self._validate(req, versioned_page)
                 if action == 'edit' and not has_collision and valid:
                     return self._do_save(req, versioned_page)
@@ -145,12 +145,19 @@ class WikiModule(Component):
                     return self._render_editor(req, page, action, has_collision)
             elif action == 'delete':
                 self._do_delete(req, versioned_page)
+            elif action == 'rename':
+                return self._do_rename(req, page)
             elif action == 'diff':
-                get_diff_options(req)
+                style, options, diff_data = get_diff_options(req)
+                contextall = diff_data['options']['contextall']
                 req.redirect(req.href.wiki(versioned_page.name, action='diff',
-                                           old_version=old_version))
+                                           old_version=old_version,
+                                           version=version,
+                                           contextall=contextall or None))
         elif action == 'delete':
-            return self._render_confirm(req, versioned_page)
+            return self._render_confirm_delete(req, versioned_page)
+        elif action == 'rename':
+            return self._render_confirm_rename(req, page)
         elif action == 'edit':
             return self._render_editor(req, versioned_page)
         elif action == 'diff':
@@ -220,7 +227,8 @@ class WikiModule(Component):
                             ignore_space_changes='-b' in diff_options)
         def version_info(v, last=0):
             return {'path': get_resource_name(self.env, page.resource),
-                    'rev': v or 'currently edited', 
+                    # TRANSLATOR: wiki page
+                    'rev': v or _('currently edited'), 
                     'shortrev': v or last + 1,
                     'href': v and req.href.wiki(page.name, version=v) or None}
         changes = [{'diffs': diffs, 'props': [],
@@ -243,15 +251,15 @@ class WikiModule(Component):
         version = int(req.args.get('version', 0)) or None
         old_version = int(req.args.get('old_version', 0)) or version
 
-        db = self.env.get_db_cnx()
-        if version and old_version and version > old_version:
-            # delete from `old_version` exclusive to `version` inclusive:
-            for v in range(old_version, version):
-                page.delete(v + 1, db)
-        else:
-            # only delete that `version`, or the whole page if `None`
-            page.delete(version, db)
-        db.commit()
+        @self.env.with_transaction()
+        def do_delete(db):
+            if version and old_version and version > old_version:
+                # delete from `old_version` exclusive to `version` inclusive:
+                for v in range(old_version, version):
+                    page.delete(v + 1, db)
+            else:
+                # only delete that `version`, or the whole page if `None`
+                page.delete(version, db)
 
         if not page.exists:
             add_notice(req, _('The page %(name)s has been deleted.',
@@ -268,6 +276,48 @@ class WikiModule(Component):
                                   version=version, name=page.name))
             req.redirect(req.href.wiki(page.name))
 
+    def _do_rename(self, req, page):
+        if page.readonly:
+            req.perm(page.resource).require('WIKI_ADMIN')
+        else:
+            req.perm(page.resource).require('WIKI_RENAME')
+ 	 
+        if 'cancel' in req.args:
+            req.redirect(get_resource_url(self.env, page.resource, req.href))
+ 	 
+        old_name, old_version = page.name, page.version
+        new_name = req.args.get('new_name', '')
+        new_name = re.sub(r'/{2,}', '/', new_name.strip('/'))
+        redirect = req.args.get('redirect')
+ 	 
+        # verify input parameters
+        warn = None
+        if not new_name:
+            warn = _('A new name is mandatory for a rename.')
+        elif not validate_page_name(new_name):
+            warn = _("The new name is invalid (a name which is separated "
+                     "with slashes cannot be '.' or '..').")
+        elif new_name == old_name:
+            warn = _('The new name must be different from the old name.')
+        elif WikiPage(self.env, new_name).exists:
+            warn = _('The page %(name)s already exists.', name=new_name)
+        if warn:
+            add_warning(req, warn)
+            return self._render_confirm_rename(req, page, new_name)
+
+        @self.env.with_transaction()
+        def do_rename(db):
+            page.rename(new_name)
+            if redirect:
+                redirection = WikiPage(self.env, old_name, db=db)
+                redirection.text = _('See [wiki:"%(name)s"].', name=new_name)
+                author = get_reporter_id(req)
+                comment = u'[wiki:"%s@%d" %s] \u2192 [wiki:"%s"].' % (
+                          new_name, old_version, old_name, new_name)
+                redirection.save(author, comment, req.remote_addr)
+        
+        req.redirect(req.href.wiki(redirect and old_name or new_name))
+
     def _do_save(self, req, page):
         if page.readonly:
             req.perm(page.resource).require('WIKI_ADMIN')
@@ -276,24 +326,23 @@ class WikiModule(Component):
         else:
             req.perm(page.resource).require('WIKI_MODIFY')
 
-        page.text = req.args.get('text')
         if 'WIKI_ADMIN' in req.perm(page.resource):
             # Modify the read-only flag if it has been changed and the user is
             # WIKI_ADMIN
             page.readonly = int('readonly' in req.args)
 
         try:
-            page.save(get_reporter_id(req, 'author'),
-                            req.args.get('comment'),
-                            req.remote_addr)
-            add_notice(req, _('Your changes have been saved.'))
+            page.save(get_reporter_id(req, 'author'), req.args.get('comment'),
+                      req.remote_addr)
+            add_notice(req, _('Your changes have been saved in version '
+                              '%(version)s.', version=page.version))
             req.redirect(get_resource_url(self.env, page.resource, req.href,
                                           version=None))
         except TracError:
             add_warning(req, _("Page not modified, showing latest version."))
             return self._render_view(req, page)
 
-    def _render_confirm(self, req, page):
+    def _render_confirm_delete(self, req, page):
         if page.readonly:
             req.perm(page.resource).require('WIKI_ADMIN')
         else:
@@ -309,8 +358,8 @@ class WikiModule(Component):
                      'num_versions': 0})
         if version is not None:
             num_versions = 0
-            for v,t,author,comment,ipnr in page.get_history():
-                num_versions += 1;
+            for v, t, author, comment, ipnr in page.get_history():
+                num_versions += 1
                 if num_versions > 1:
                     break
             data.update({'new_version': version, 'old_version': old_version,
@@ -318,6 +367,17 @@ class WikiModule(Component):
         self._wiki_ctxtnav(req, page)
         return 'wiki_delete.html', data, None
 
+    def _render_confirm_rename(self, req, page, new_name=None):
+        if page.readonly:
+            req.perm(page.resource).require('WIKI_ADMIN')
+        else:
+            req.perm(page.resource).require('WIKI_RENAME')
+           
+        data = self._page_data(req, page, 'rename')
+        data['new_name'] = new_name is None and page.name or new_name
+        self._wiki_ctxtnav(req, page)
+        return 'wiki_rename.html', data, None
+        
     def _render_diff(self, req, page):
         if not page.exists:
             raise TracError(_('Version %(num)s of page "%(name)s" does not '
@@ -340,7 +400,6 @@ class WikiModule(Component):
 
         date = author = comment = ipnr = None
         num_changes = 0
-        old_page = None
         prev_version = next_version = None
         for version, t, a, c, i in latest_page.get_history():
             if version == new_version:
@@ -353,19 +412,18 @@ class WikiModule(Component):
                     num_changes += 1
                     if not prev_version:
                         prev_version = version
-                    if (old_version and version == old_version) or \
-                            not old_version:
+                    if old_version is None or version == old_version:
                         old_version = version
-                        old_page = WikiPage(self.env, page.name, old_version)
-                        req.perm(old_page.resource).require('WIKI_VIEW')
                         break
                 else:
                     next_version = version
         if not old_version:
             old_version = 0
+        old_page = WikiPage(self.env, page.name, old_version)
+        req.perm(old_page.resource).require('WIKI_VIEW')
 
         # -- text diffs
-        old_text = old_page and old_page.text.splitlines() or []
+        old_text = old_page.text.splitlines()
         new_text = page.text.splitlines()
         diff_data, changes = self._prepare_diff(req, page, old_text, new_text,
                                                 old_version, new_version)
@@ -393,7 +451,8 @@ class WikiModule(Component):
             'changes': changes,
             'diff': diff_data,
         })
-        prevnext_nav(req, _('Change'), _('Wiki History'))
+        prevnext_nav(req, _('Previous Change'), _('Next Change'), 
+                     _('Wiki History'))
         return 'wiki_diff.html', data, None
 
     def _render_editor(self, req, page, action='edit', has_collision=False):
@@ -417,25 +476,42 @@ class WikiModule(Component):
             if template_page and template_page.exists and \
                    'WIKI_VIEW' in req.perm(template_page.resource):
                 page.text = template_page.text
-        if action == 'preview':
+        if action in ('preview', 'diff'):
             page.readonly = 'readonly' in req.args
 
         author = get_reporter_id(req, 'author')
         comment = req.args.get('comment', '')
-        editrows = req.args.get('editrows')
-        
-        if editrows:
-            pref = req.session.get('wiki_editrows', '20')
-            if editrows != pref:
-                req.session['wiki_editrows'] = editrows
+
+        defaults = {'editrows': 20}
+        prefs = dict((key, req.session.get('wiki_%s' % key, defaults.get(key)))
+                     for key in ('editrows', 'sidebyside'))
+
+        if 'from_editor' in req.args:
+            sidebyside = req.args.get('sidebyside') or None
+            if sidebyside != prefs['sidebyside']:
+                if sidebyside:
+                    req.session['wiki_sidebyside'] = '1'
+                else:
+                    del req.session['wiki_sidebyside']
         else:
-            editrows = req.session.get('wiki_editrows', '20')
+            sidebyside = prefs['sidebyside']
+
+        if sidebyside:
+            editrows = max(int(prefs['editrows']), 
+                           len(page.text.splitlines()) + 1)
+        else:
+            editrows = req.args.get('editrows')
+            if editrows:
+                if editrows != prefs['editrows']:
+                    req.session['wiki_editrows'] = editrows
+            else:
+                editrows = prefs['editrows']
 
         data = self._page_data(req, page, action)
         data.update({
             'author': author,
             'comment': comment,
-            'edit_rows': editrows,
+            'edit_rows': editrows, 'sidebyside': sidebyside,
             'scroll_bar_pos': req.args.get('scroll_bar_pos', ''),
             'diff': None,
         })
@@ -447,8 +523,12 @@ class WikiModule(Component):
             data.update({'diff': diff_data, 'changes': changes,
                          'action': 'preview', 'merge': action == 'merge',
                          'longcol': 'Version', 'shortcol': 'v'})
+        elif sidebyside and action != 'collision':
+            data['action'] = 'preview'
         
         self._wiki_ctxtnav(req, page)
+        Chrome(self.env).add_wiki_toolbars(req)
+        Chrome(self.env).add_auto_preview(req)
         return 'wiki_edit.html', data, None
 
     def _render_history(self, req, page):
@@ -472,7 +552,8 @@ class WikiModule(Component):
                 'ipnr': ipnr
             })
         data.update({'history': history, 'resource': page.resource})
-        add_ctxtnav(req, 'Back to '+page.name, req.href.wiki(page.name))
+        add_ctxtnav(req, _("Back to %(wikipage)s", wikipage=page.name),
+                           req.href.wiki(page.name))
         return 'history_view.html', data, None
 
     def _render_view(self, req, page):
@@ -494,10 +575,31 @@ class WikiModule(Component):
         if page.name == 'WikiStart':
             data['title'] = ''
 
+        ws = WikiSystem(self.env)
+        context = Context.from_request(req, page.resource)
+        higher, related = [], []
         if not page.exists:
             if 'WIKI_CREATE' not in req.perm(page.resource):
                 raise ResourceNotFound(_('Page %(name)s not found',
                                          name=page.name))
+            formatter = OneLinerFormatter(self.env, context)
+            if '/' in page.name:
+                parts = page.name.split('/')
+                for i in range(len(parts) - 2, -1, -1):
+                    name = '/'.join(parts[:i] + [parts[-1]])
+                    if not ws.has_page(name):
+                        higher.append(ws._format_link(formatter, 'wiki',
+                                                    '/' + name, name, False))
+            else:
+                name = page.name
+            name = name.lower()
+            related = [each for each in ws.pages
+                       if name in each.lower()
+                          and 'WIKI_VIEW' in req.perm('wiki', each)]
+            related.sort()
+            related = [ws._format_link(formatter, 'wiki', '/' + each, each,
+                                       False)
+                       for each in related]
 
         latest_page = WikiPage(self.env, page.name, version=None)
         req.perm(latest_page.resource).require('WIKI_VIEW')
@@ -519,9 +621,9 @@ class WikiModule(Component):
                 version = None
             
         prefix = self.PAGE_TEMPLATES_PREFIX
-        templates = [template[len(prefix):] for template in
-                     WikiSystem(self.env).get_pages(prefix) if
-                     'WIKI_VIEW' in req.perm('wiki', template)]
+        templates = [template[len(prefix):]
+                     for template in ws.get_pages(prefix)
+                     if 'WIKI_VIEW' in req.perm('wiki', template)]
 
         # -- prev/up/next links
         if prev_version:
@@ -545,24 +647,31 @@ class WikiModule(Component):
 
         # Add ctxtnav entries
         if version:
-            prevnext_nav(req, _('Version'), _('View Latest Version'))
-            add_ctxtnav(req, _('Last Change'),
-                        req.href.wiki(page.name, action='diff',
-                                      version=page.version))
+            prevnext_nav(req, _('Previous Version'), _('Next Version'),
+                         _('View Latest Version'))
         else:
             if parent:
                 add_ctxtnav(req, _('Up'), req.href.wiki(parent))
             self._wiki_ctxtnav(req, page)
 
-        context = Context.from_request(req, page.resource)
+        # Plugin content validation
+        fields = {'text': page.text}
+        for manipulator in self.page_manipulators:
+            manipulator.prepare_wiki_page(req, page, fields)
+        text = fields.get('text', '')
+
         data.update({
             'context': context,
+            'text': text,
             'latest_version': latest_page.version,
             'attachments': AttachmentModule(self.env).attachment_data(context),
             'default_template': self.DEFAULT_PAGE_TEMPLATE,
             'templates': templates,
-            'version': version
+            'version': version,
+            'higher': higher, 'related': related,
+            'resourcepath_template': 'wiki_page_path.html',
         })
+        add_script(req, 'common/js/folding.js')
         return 'wiki_view.html', data, None
     
     def _wiki_ctxtnav(self, req, page):
@@ -572,9 +681,6 @@ class WikiModule(Component):
         if page.exists:
             add_ctxtnav(req, _('History'), req.href.wiki(page.name, 
                                                          action='history'))
-            add_ctxtnav(req, _('Last Change'),
-                        req.href.wiki(page.name, action='diff',
-                                      version=page.version))
 
     # ITimelineEventProvider methods
 
@@ -589,12 +695,12 @@ class WikiModule(Component):
             cursor = db.cursor()
             cursor.execute("SELECT time,name,comment,author,version "
                            "FROM wiki WHERE time>=%s AND time<=%s",
-                           (to_timestamp(start), to_timestamp(stop)))
-            for ts,name,comment,author,version in cursor:
+                           (to_utimestamp(start), to_utimestamp(stop)))
+            for ts, name, comment, author, version in cursor:
                 wiki_page = wiki_realm(id=name, version=version)
                 if 'WIKI_VIEW' not in req.perm(wiki_page):
                     continue
-                yield ('wiki', datetime.fromtimestamp(ts, utc), author,
+                yield ('wiki', from_utimestamp(ts), author,
                        (wiki_page, comment))
 
             # Attachments
@@ -607,15 +713,19 @@ class WikiModule(Component):
         if field == 'url':
             return context.href.wiki(wiki_page.id, version=wiki_page.version)
         elif field == 'title':
-            return tag(tag.em(get_resource_name(self.env, wiki_page)),
-                       wiki_page.version > 1 and ' edited' or ' created')
+            name = tag.em(get_resource_name(self.env, wiki_page))
+            if wiki_page.version > 1:
+                return tag_('%(page)s edited', page=name)
+            else:
+                return tag_('%(page)s created', page=name)
         elif field == 'description':
             markup = format_to(self.env, None, context(resource=wiki_page),
                                comment)
             if wiki_page.version > 1:
                 diff_href = context.href.wiki(
                     wiki_page.id, version=wiki_page.version, action='diff')
-                markup = tag(markup, ' ', tag.a('(diff)', href=diff_href))
+                markup = tag(markup,
+                             ' (', tag.a(_('diff'), href=diff_href), ')')
             return markup
 
     # ISearchSource methods
@@ -644,7 +754,7 @@ class WikiModule(Component):
             if 'WIKI_VIEW' in req.perm(page):
                 yield (get_resource_url(self.env, page, req.href),
                        '%s: %s' % (name, shorten_line(text)),
-                       datetime.fromtimestamp(ts, utc), author,
+                       from_utimestamp(ts), author,
                        shorten_result(text, terms))
         
         # Attachments
